#
# Package tooltip - creates floating tips for widget
# tips can display
# 1. Static text set upon tip creation
# 2. Contents of specified variable
# 3. Result of command execution
# 
# Tooltips are really top-level windows with class Tooltip named
# $widget.tooltip
#

# Uses resources *Tooltip*Font
# *Tooltip*Foreground
# *Tooltip*Background
# *Tooltip.Delay (popup delay, ms)

# Default options

package require Tk

option set *Tooltip*Font {Fixed 10} widgetDefault
option set *Tooltip*Background yellow widgetDefault
option set *Tooltip*Foreground black widgetDefault
option set *Tooltip.BorderWidth 1 widgetDefault
option set *Tooltip.Delay 2000 widgetDefault

#
# Syntax: 
#   tooltip widget ?-text "Text" | -variable varname | -command command?
# 
#

namespace eval tooltip {
	proc tooltip {widget args} {
		variable tipcommand
		if [winfo exist $widget.tooltip] {
			clearTipCommand $widget
			setupTipLabel $widget $args
		} else {
			toplevel $widget.tooltip -class Tooltip
			label $widget.tooltip.l
			pack $widget.tooltip.l
			wm withdraw $widget.tooltip
			wm overrideredirect $widget.tooltip
			setupBindings $widget
			setupTipLabel $widget $args
		}
	}	
	proc clearTipCommand widget {
		variable tipcommand
		if [info exists tipcommand($widget)] {
			unset tipcommand($widget.tooltip)
		}
	}


	proc setupBindings {widget} {
		bind $widget <Enter> {+::tooltip::start %W}
		bind $widget <Leave> {+::tooltip::cancel %W}
		bind $widget <Motion> {+::tooltip::reset %W}
		bind $widget <Destroy> {+::tooltip::cancel %W;
					::tooltip::clearTipCommand %W}
	}	
	
	proc start {widget} {
		variable afterId
		set afterId($widget) [after [option get $widget.tooltip delay Delay] [list ::tooltip::show $widget]]
	}

	proc cancel {widget} {
		variable afterId
		if {[info exists afterId($widget)]} {
			after cancel $afterId($widget)
			unset afterId($widget)
		}
		if {[wm state $widget.tooltip]=="normal"} {
			wm withdraw $widget.tooltip
		}
	}

	proc show {widget} {
		variable tipcommand
		if [info exists tipcommand($widget)] {
			$widget.tooltip.l configure -text [uplevel $tipcommand($widget)]
		}
		wm geometry $widget.tooltip +[expr [winfo pointerx $widget]+2]+[expr [winfo pointery $widget]+2]
		wm deiconify $widget.tooltip
		raise $widget.tooltip
	}

	proc reset {widget} {
		cancel $widget
		start $widget
	}
	proc setupTipLabel {widget arglist} {
		variable tipcommand
		foreach {type value} $arglist break
		switch -glob -- $type {
			-text {$widget.tooltip.l configure -text $value}
			-var* {$widget.tooltip.l configure -textvar $value}
			-comm* {set tipcommand($widget) $value}
			default {
				return -code error "Invalid option should be one of -text -variable -command"
			}
		}
	}
	namespace export tooltip
}
	namespace import tooltip::*

package provide tooltip 0.1
